import time
from typing import Dict, List, Optional

from nonebot.adapters.onebot.v11 import Bot
from pydantic import BaseModel

from nekro_agent.core.bot import get_bot
from nekro_agent.core.config import config
from nekro_agent.core.logger import logger
from nekro_agent.schemas.chat_message import ChatType

MAX_PRESET_STATUS_LIST_SIZE = 99
MAX_PRESET_STATUS_SHOW_SIZE = 6


class PresetStatus(BaseModel):
    """预设状态"""

    setting_name: str
    description: str
    translated_timestamp: int

    def render_prompts(self, extra: bool = False) -> str:
        time_diff = time.time() - self.translated_timestamp
        time_diff_str = time.strftime("%H:%M:%S", time.gmtime(time_diff))
        addition_str = (
            (
                " Please Use `update_preset_status` to update it."
                if time_diff > 300
                else " Use `update_preset_status` to update it If doesn't fit the current scene description."
            )
            if extra
            else ""
        )
        return f"{self.setting_name}: {self.description} (updated {time_diff_str} ago.{addition_str})"


class PresetEffect(BaseModel):
    """预设效果"""

    effect_name: str
    description: str
    start_time: int
    duration: int

    @classmethod
    def create(cls, effect_name: str, description: str, duration: int = 0):
        start_time = int(time.time())
        return cls(
            effect_name=effect_name,
            description=description,
            start_time=start_time,
            duration=duration,
        )

    def render_prompts(self) -> str:
        time_diff = time.time() - self.start_time
        time_diff_str = time.strftime("%H:%M:%S", time.gmtime(time_diff))
        return f"{self.effect_name}: {self.description} (started {time_diff_str} ago. `remove_effect` if expired.)"


class ChannelData(BaseModel):
    """聊天频道数据"""

    chat_key: str
    preset_status_list: List[PresetStatus] = []
    preset_effects: Dict[str, PresetEffect] = {}

    class Config:
        extra = "ignore"

    def _append_preset_status(self, preset_status: PresetStatus):
        """添加预设状态"""
        self.preset_status_list.append(preset_status)
        if len(self.preset_status_list) > MAX_PRESET_STATUS_LIST_SIZE:
            self.preset_status_list.pop(0)

    async def update_preset_status(self, preset_status: PresetStatus):
        """更新预设状态"""
        latest_preset_status: Optional[PresetStatus] = self.get_latest_preset_status()
        if latest_preset_status is not None and latest_preset_status.setting_name == preset_status.setting_name:
            self.preset_status_list[-1].translated_timestamp = preset_status.translated_timestamp
        else:
            self._append_preset_status(preset_status)
            chat_type, chat_id = self.chat_key.split("_")
            try:
                if chat_type == "group" and config.SESSION_ENABLE_CHANGE_NICKNAME:
                    await get_bot().set_group_card(
                        group_id=int(chat_id),
                        user_id=int(config.BOT_QQ),
                        card=f"{config.SESSION_NICKNAME_PREFIX}{preset_status.setting_name}",
                    )
            except Exception as e:
                logger.warning(f"会话 {self.chat_key} 尝试更新群名片失败: {e}")

    def get_latest_preset_status(self) -> Optional[PresetStatus]:
        if len(self.preset_status_list) == 0:
            return None
        return self.preset_status_list[-1]

    async def update_preset_effect(self, effect: PresetEffect):
        """更新预设效果"""
        self.preset_effects[effect.effect_name] = effect

    async def remove_preset_effect(self, effect_name: str, fuzzy: bool = False) -> bool:
        """移除预设效果"""
        if effect_name in self.preset_effects:
            del self.preset_effects[effect_name]
            return True
        if fuzzy:
            for effect in self.preset_effects:
                if effect_name.replace(" ", "") in effect.replace(" ", ""):
                    del self.preset_effects[effect]
                    return True
        return False

    def render_prompts(self) -> str:
        """渲染提示词"""
        latest_preset_status = self.get_latest_preset_status()
        if latest_preset_status is None:
            return "Current Character Setting status: No special status. (Use `update_preset_status` to update it **immediately**!)"

        history_str: str = ""
        if len(self.preset_status_list) > 1:
            history_str = "History Status:\n " + "".join(
                [
                    f"{preset_status.render_prompts()}\n"
                    for preset_status in self.preset_status_list[-MAX_PRESET_STATUS_SHOW_SIZE:-1]
                ],
            )

        effect_str: str = (
            "".join(
                [f"- {effect.render_prompts()}\n" for effect in list(self.preset_effects.values())],
            )
            if len(self.preset_effects) > 0
            else "No effects. (use `set_effect` to add one)"
        )

        return (
            f"{history_str}"
            "Current Character Setting status:"
            f"{latest_preset_status.render_prompts(extra=True)}\n\n"
            "Current Effects:\n"
            f"{effect_str}"
        )

    async def clear_status(self):
        self.preset_status_list = []
        chat_type, chat_id = self.chat_key.split("_")
        if chat_type == "group" and config.SESSION_ENABLE_CHANGE_NICKNAME:
            await get_bot().set_group_card(
                group_id=int(chat_id),
                user_id=int(config.BOT_QQ),
                card=f"{config.SESSION_NICKNAME_PREFIX}{config.AI_CHAT_PRESET_NAME}",
            )

    @property
    def chat_type(self) -> ChatType:
        """获取聊天频道类型"""
        return ChatType.from_chat_key(self.chat_key)



"""用户相关 API

此模块提供了与用户相关的 API 接口。
"""

from nekro_agent.api.schemas import AgentCtx
from nekro_agent.tools.common_util import (
    convert_file_name_to_container_path,
    download_file,
)

__all__ = [
    "get_avatar",
]


async def get_avatar(user_qq: str, ctx: AgentCtx) -> str:
    """获取用户头像

    Args:
        user_qq (str): 用户QQ号
        ctx (AgentCtx): 上下文对象

    Returns:
        str: 头像文件路径

    Example:
        ```python
        from nekro_agent.api.user import get_avatar

        # 获取用户头像
        avatar_path = get_avatar("123456789", ctx)
        ```
    """
    try:
        file_path, file_name = await download_file(
            f"https://q1.qlogo.cn/g?b=qq&nk={user_qq}&s=640",
            from_chat_key=ctx.from_chat_key,
        )
        return str(convert_file_name_to_container_path(file_name))
    except Exception as e:
        raise Exception(f"获取用户头像失败: {e}") from e



import json
from typing import AsyncGenerator, Dict, List

from fastapi import APIRouter, Body, Depends, HTTPException
from sse_starlette.sse import EventSourceResponse

from nekro_agent.core import logger
from nekro_agent.core.os_env import OsEnv
from nekro_agent.schemas.message import Ret
from nekro_agent.services.extension import (
    apply_extension_code,
    delete_ext_file,
    generate_extension_code,
    generate_extension_code_stream,
    generate_extension_template,
    get_ext_workdir_files,
    read_ext_file,
    reload_ext_workdir,
    save_ext_file,
)
from nekro_agent.services.extension.manager import get_all_ext_meta_data
from nekro_agent.systems.user.deps import get_current_active_user
from nekro_agent.tools.collector import MethodType, agent_collector

router = APIRouter(prefix="/extensions", tags=["Extensions"])


@router.get("", summary="获取扩展列表")
async def get_extensions(_=Depends(get_current_active_user)) -> Ret:
    """获取所有已加载的扩展信息"""
    extensions = []

    # 获取所有方法及其所属的标签
    all_methods = {}
    for _tag, methods in agent_collector.tag_map.items():
        for method in methods:
            if method.__doc__:
                try:
                    method_type = agent_collector.get_method_type(method)
                    # 使用方法所在模块的文件名作为扩展名
                    ext_name = method.__module__.split(".")[-1]
                    if ext_name not in all_methods:
                        all_methods[ext_name] = []
                    all_methods[ext_name].append(
                        {"name": method.__name__, "type": method_type, "description": method.__doc__.strip()},
                    )
                except ValueError:
                    continue

    # 为每个扩展添加其对应的方法
    for ext in get_all_ext_meta_data():
        if ext.name not in all_methods:
            logger.warning(f"扩展: {ext.name} 未找到挂载方法列表，请确认扩展元数据与包名是否对应")
        extensions.append(
            {
                "name": ext.name,
                "version": ext.version,
                "description": ext.description,
                "author": ext.author,
                "methods": all_methods.get(ext.name, []),  # 获取该扩展对应的方法列表
                "is_enabled": True,
            },
        )

    return Ret.success(msg="获取扩展列表成功", data=extensions)


@router.get("/files", summary="获取扩展文件列表")
async def get_extension_files(_=Depends(get_current_active_user)) -> Ret:
    """获取工作目录下的所有扩展文件"""
    try:
        files = get_ext_workdir_files()
        return Ret.success(msg="获取扩展文件列表成功", data=files)
    except Exception as e:
        return Ret.error(msg=f"获取扩展文件列表失败: {e!s}")


@router.get("/file/{file_path:path}", summary="获取扩展文件内容")
async def get_extension_file(file_path: str, _=Depends(get_current_active_user)) -> Ret:
    """获取指定扩展文件的内容"""
    try:
        content = read_ext_file(file_path)
        return Ret.success(msg="获取扩展文件内容成功", data={"content": content})
    except Exception as e:
        return Ret.error(msg=f"获取扩展文件内容失败: {e!s}")


@router.post("/file/{file_path:path}", summary="保存扩展文件内容")
async def save_extension_file(file_path: str, content: str = Body(...), _=Depends(get_current_active_user)) -> Ret:
    """保存扩展文件内容"""
    try:
        save_ext_file(file_path, content)
        return Ret.success(msg="保存扩展文件内容成功")
    except Exception as e:
        return Ret.error(msg=f"保存扩展文件内容失败: {e!s}")


@router.post("/generate", summary="生成扩展代码")
async def generate_extension(
    file_path: str = Body(..., description="文件路径"),
    prompt: str = Body(..., description="提示词"),
    current_code: str = Body(None, description="当前代码"),
    _=Depends(get_current_active_user),
) -> Ret:
    """生成扩展代码"""
    try:
        generated_code = await generate_extension_code(file_path, prompt, current_code)
        return Ret.success(msg="生成代码成功", data={"code": generated_code})
    except Exception as e:
        return Ret.error(msg=f"生成代码失败: {e!s}")


@router.post("/template", summary="生成扩展模板")
async def create_extension_template(
    name: str = Body(..., description="扩展名称"),
    description: str = Body(..., description="扩展描述"),
    _=Depends(get_current_active_user),
) -> Ret:
    """生成扩展模板"""
    try:
        template = generate_extension_template(name, description)
        return Ret.success(msg="生成扩展模板成功", data={"template": template})
    except Exception as e:
        return Ret.error(msg=f"生成扩展模板失败: {e!s}")


@router.post("/generate/stream", summary="流式生成扩展代码")
async def stream_generate_extension(
    file_path: str = Body(..., description="文件路径"),
    prompt: str = Body(..., description="提示词"),
    current_code: str | None = Body(None, description="当前代码"),
    _=Depends(get_current_active_user),
) -> EventSourceResponse:
    """流式生成扩展代码"""

    async def event_generator() -> AsyncGenerator[str, None]:
        try:
            async for chunk in generate_extension_code_stream(file_path, prompt, current_code):
                yield f'{{"type": "content", "content": {json.dumps(chunk)}}}\n\n'
            yield '{"type": "done"}\n\n'
        except Exception as e:
            logger.error(f"生成代码失败: {e!s}")
            yield f'{{"type": "error", "error": {json.dumps(str(e))}}}\n\n'

    return EventSourceResponse(event_generator())


@router.post("/apply", summary="应用生成的代码")
async def apply_extension(
    file_path: str = Body(..., description="文件路径"),
    prompt: str = Body(..., description="提示词"),
    current_code: str = Body(..., description="当前代码"),
    _=Depends(get_current_active_user),
) -> Ret:
    """应用生成的代码"""
    try:
        applied_code = await apply_extension_code(file_path, prompt, current_code)
        return Ret.success(msg="应用代码成功", data={"code": applied_code})
    except Exception as e:
        return Ret.error(msg=f"应用代码失败: {e!s}")


@router.get("/files")
async def list_extension_files() -> List[str]:
    """获取所有扩展文件"""
    return get_ext_workdir_files()


@router.delete("/files/{file_path:path}")
async def delete_extension_file(file_path: str) -> Dict[str, str]:
    """删除扩展文件"""
    try:
        delete_ext_file(file_path)
    except (ValueError, FileNotFoundError) as e:
        raise HTTPException(status_code=400, detail=str(e)) from e
    else:
        return {"message": "文件删除成功"}


@router.get("/files/{file_path:path}")
async def read_extension_file(file_path: str) -> str:
    """读取扩展文件内容"""
    try:
        return read_ext_file(file_path)
    except (ValueError, FileNotFoundError) as e:
        raise HTTPException(status_code=400, detail=str(e)) from e


@router.post("/reload", summary="重载所有扩展")
async def reload_extensions(_=Depends(get_current_active_user)) -> Ret:
    """重载所有扩展"""
    try:
        reload_ext_workdir()
        return Ret.success(msg="重载扩展成功")
    except Exception as e:
        return Ret.error(msg=f"重载扩展失败: {e!s}")



"""消息相关 API

此模块提供了与消息发送相关的 API 接口。
"""

from nekro_agent.api.schemas import AgentCtx
from nekro_agent.schemas.agent_message import (
    AgentMessageSegment,
    AgentMessageSegmentType,
)
from nekro_agent.services.chat import chat_service
from nekro_agent.tools.common_util import (
    convert_file_name_to_container_path,
    download_file,
)

__all__ = [
    "download_from_url",
    "send_file",
    "send_image",
    "send_text",
]


async def send_text(chat_key: str, message: str, ctx: AgentCtx, *, record: bool = True) -> None:
    """发送文本消息

    Args:
        chat_key (str): 会话标识，格式为 "{type}_{id}"，例如 "group_123456"
        message (str): 要发送的文本消息
        ctx (AgentCtx): 上下文对象
        record (bool, optional): 是否记录到上下文。默认为 True

    Example:
        ```python
        from nekro_agent.api.message import send_text

        # 发送文本消息到群组（记录到上下文）
        send_text("group_123456", "你好，世界！", ctx)

        # 发送文本消息到群组（不记录到上下文）
        send_text("group_123456", "这是一条临时消息", ctx, record=False)
        ```
    """
    message_ = [AgentMessageSegment(content=message)]
    try:
        await chat_service.send_agent_message(chat_key, message_, ctx, record=record)
    except Exception as e:
        raise Exception("发送文本消息失败: 请确保会话标识正确且内容不为空或过长") from e


async def send_file(chat_key: str, file_path: str, ctx: AgentCtx, *, record: bool = True) -> None:
    """发送文件消息

    Args:
        chat_key (str): 会话标识，格式为 "{type}_{id}"，例如 "group_123456"
        file_path (str): 文件路径或URL
        ctx (AgentCtx): 上下文对象
        record (bool, optional): 是否记录到上下文。默认为 True

    Example:
        ```python
        from nekro_agent.api.message import send_file

        # 发送文件（记录到上下文）
        send_file("group_123456", "/path/to/file.pdf", ctx)

        # 发送文件（不记录到上下文）
        send_file("group_123456", "/path/to/temp.pdf", ctx, record=False)
        ```
    """
    message_ = [AgentMessageSegment(type=AgentMessageSegmentType.FILE, content=file_path)]
    try:
        await chat_service.send_agent_message(chat_key, message_, ctx, file_mode=True, record=record)
    except Exception as e:
        raise Exception(f"发送文件消息失败: {e}") from e


async def send_image(chat_key: str, image_path: str, ctx: AgentCtx, *, record: bool = True) -> None:
    """发送图片消息

    Args:
        chat_key (str): 会话标识，格式为 "{type}_{id}"，例如 "group_123456"
        image_path (str): 图片路径或URL
        ctx (AgentCtx): 上下文对象
        record (bool, optional): 是否记录到上下文。默认为 True

    Example:
        ```python
        from nekro_agent.api.message import send_image

        # 发送图片（记录到上下文）
        send_image("group_123456", "/path/to/image.jpg", ctx)

        # 发送图片（不记录到上下文）
        send_image("group_123456", "/path/to/temp.jpg", ctx, record=False)
        ```
    """
    message_ = [AgentMessageSegment(type=AgentMessageSegmentType.FILE, content=image_path)]
    try:
        await chat_service.send_agent_message(chat_key, message_, ctx, record=record)
    except Exception as e:
        raise Exception(f"发送图片消息失败: {e}") from e


async def download_from_url(url: str, ctx: AgentCtx) -> str:
    """从URL下载文件

    Args:
        url (str): 文件URL
        ctx (AgentCtx): 上下文对象

    Returns:
        str: 下载后的文件路径

    Example:
        ```python
        from nekro_agent.api.message import download_from_url

        # 下载文件
        file_path = download_from_url("https://example.com/file.jpg", ctx)
        ```
    """
    try:
        file_path, file_name = await download_file(url, from_chat_key=ctx.from_chat_key)
        return str(convert_file_name_to_container_path(file_name))
    except Exception as e:
        raise Exception(f"下载文件失败: {e}") from e



import asyncio
import time
from datetime import datetime
from typing import Awaitable, Callable, Dict, List, Optional

from nekro_agent.core import logger
from nekro_agent.services.message.message_service import message_service


class TimerTask:
    """定时任务类"""

    def __init__(self, chat_key: str, trigger_time: int, event_desc: str):
        self.chat_key = chat_key
        self.trigger_time = trigger_time
        self.event_desc = event_desc
        self.task: Optional[asyncio.Task] = None
        self.temporary: bool = False  # 是否为临时定时器
        self.callback: Optional[Callable[[], Awaitable[None]]] = None  # 回调函数


class TimerService:
    """定时器服务类"""

    def __init__(self):
        self.tasks: Dict[str, List[TimerTask]] = {}  # chat_key -> [TimerTask]
        self.running = False

    async def start(self):
        """启动定时器服务"""
        if self.running:
            return
        self.running = True
        asyncio.create_task(self._timer_loop())
        logger.info("Timer service started")

    async def stop(self):
        """停止定时器服务"""
        self.running = False
        # 取消所有任务
        for tasks in self.tasks.values():
            for task in tasks:
                if task.task and not task.task.done():
                    task.task.cancel()
        self.tasks.clear()
        logger.info("Timer service stopped")

    async def set_timer(
        self,
        chat_key: str,
        trigger_time: int,
        event_desc: str,
        silent: bool = False,
        override: bool = False,
        temporary: Optional[bool] = None,
        callback: Optional[Callable[[], Awaitable[None]]] = None,
    ) -> bool:
        """设置定时器

        Args:
            chat_key (str): 会话标识
            trigger_time (int): 触发时间戳。如果为0则立即触发会话;如果小于0则清空当前会话的定时器。
            event_desc (str): 详细事件描述
            silent (bool, optional): 是否静默设置. Defaults to False.
            override (bool): 是否为临时定时器，每个会话只能存在一个临时定时器，新设置的临时定时器会自动覆盖同一会话中已存在的临时定时器。
            temporary (Optional[bool], optional): 清除定时器时的类型筛选。None表示清除所有定时器，True只清除临时定时器，False只清除非临时定时器。仅在 trigger_time < 0 时生效。
            callback (Optional[Callable[[], Awaitable[None]]], optional): 定时器触发时执行的回调函数. Defaults to None.

        Returns:
            bool: 是否设置成功
        """
        # 如果触发时间小于0，清空当前会话的定时器
        if trigger_time < 0:
            if chat_key in self.tasks:
                if temporary is None:
                    # 清空所有定时器
                    del self.tasks[chat_key]
                else:
                    # 根据 temporary 参数筛选要保留的定时器
                    self.tasks[chat_key] = [task for task in self.tasks[chat_key] if task.temporary != temporary]
                    if not self.tasks[chat_key]:
                        del self.tasks[chat_key]
                if not silent:
                    logger.info(f"已清空会话 {chat_key} 的{'所有' if temporary is None else '临时' if temporary else '非临时'}定时器")
            return True

        # 如果触发时间为0，立即触发会话
        if trigger_time == 0:
            await message_service.schedule_agent_task(chat_key)
            return True

        # 检查触发时间是否已过
        if trigger_time <= int(time.time()):
            return False

        if chat_key not in self.tasks:
            self.tasks[chat_key] = []
        elif override:
            # 如果是临时定时器，移除之前的临时定时器
            self.tasks[chat_key] = [task for task in self.tasks[chat_key] if not task.temporary]

        # 创建定时任务
        task = TimerTask(chat_key, trigger_time, event_desc)
        task.temporary = override
        task.callback = callback
        self.tasks[chat_key].append(task)
        if not silent:
            logger.info(f"定时器设置成功: {chat_key} | 触发时间: {datetime.fromtimestamp(trigger_time)}")
        return True

    async def _timer_loop(self):
        """定时器循环"""
        while self.running:
            current_time = int(time.time())

            # 检查所有任务
            for chat_key, tasks in list(self.tasks.items()):
                triggered_tasks = []
                for task in tasks:
                    if task.trigger_time <= current_time:
                        triggered_tasks.append(task)
                        # 执行回调函数或发送系统消息
                        if task.callback:
                            await task.callback()
                        elif task.event_desc:
                            system_message = f"⏰ 定时提醒：{task.event_desc}"
                            await message_service.push_system_message(
                                chat_key=task.chat_key,
                                agent_messages=system_message,
                                trigger_agent=True,
                            )
                        else:
                            await message_service.schedule_agent_task(task.chat_key)

                # 移除已触发的任务
                self.tasks[chat_key] = [t for t in tasks if t not in triggered_tasks]
                if not self.tasks[chat_key]:
                    del self.tasks[chat_key]

            await asyncio.sleep(1)  # 每秒检查一次


# 全局定时器服务实例
timer_service = TimerService()



from enum import Enum
from typing import List

from pydantic import BaseModel


class AgentMessageSegmentType(str, Enum):
    """消息段类型枚举"""

    TEXT = "text"
    FILE = "file"


class AgentMessageSegment(BaseModel):
    """由 Agent 发送消息使用的消息段

    Attributes:
        type: 消息段类型
        content: 消息段内容

    Example:
        ```python
        AgentMessageSegment(type=AgentMessageSegmentType.TEXT, content="Hello, world!")  # 文本消息段
        AgentMessageSegment(type=AgentMessageSegmentType.FILE, content="shared/file.txt")  # 文件消息段
        ```
    """

    type: AgentMessageSegmentType = AgentMessageSegmentType.TEXT
    content: str

    class Config:
        use_enum_values = True

    def get_prompt(self):
        if self.type == AgentMessageSegmentType.TEXT:
            return f"{self.content}"

        if self.type == AgentMessageSegmentType.FILE:
            return f"[File: {self.content}]"

        return ""


def convert_agent_message_to_prompt(agent_messages: List[AgentMessageSegment]) -> str:
    prompt = ""
    for message in agent_messages:
        prompt += message.get_prompt() + " "

    return prompt



"""扩展加载器

负责扩展的加载和重载功能。
"""

import sys
from importlib import import_module, reload
from pathlib import Path
from typing import List

from nekro_agent.core.config import config
from nekro_agent.core.logger import logger
from nekro_agent.core.os_env import EXT_WORKDIR
from nekro_agent.tools.collector import agent_collector

from .models import ExtMetaData

# 存储已加载的扩展元数据
BUILTIN_EXT_META_DATA: List[ExtMetaData] = []
WORKDIR_EXT_META_DATA: List[ExtMetaData] = []
reloaded_modules: List[str] = []

# 初始化工作目录
ext_workdir = Path(EXT_WORKDIR)
ext_workdir.mkdir(parents=True, exist_ok=True)

# 将 ext_workdir 添加到 Python 模块搜索路径
ext_workdir_str = str(ext_workdir.absolute())
if ext_workdir_str not in sys.path:
    sys.path.insert(0, ext_workdir_str)


def init_extensions() -> None:
    """初始化并加载所有内置扩展"""
    for module_path in config.EXTENSION_MODULES:
        logger.info(f"正在加载扩展模块: {module_path}")
        try:
            module = import_module(module_path)
        except ImportError:
            logger.error(f"从 {module_path} 加载扩展模块失败")
            continue

        if hasattr(module, "__meta__"):
            module_meta_data = module.__meta__

            if isinstance(module_meta_data, ExtMetaData):
                logger.success(
                    f'从 "{module_path}" 扩展模块: "{module_meta_data.name}" by "{module_meta_data.author or "Unknown"}" 加载验证成功',
                )
                BUILTIN_EXT_META_DATA.append(module_meta_data)
            else:
                logger.warning(f'加载扩展模块: "{module_path}" 元数据格式错误')
        else:
            logger.warning(f'加载扩展模块: "{module_path}" 缺少元数据')


def reload_ext_workdir() -> List[str]:
    """重载工作目录下的所有扩展模块

    Returns:
        List[str]: 重载的模块列表
    """
    # 清空工作目录扩展元数据列表
    WORKDIR_EXT_META_DATA.clear()

    # 从所有标签中移除工作目录模块的方法
    for tag, methods in agent_collector.tag_map.items():
        # 创建一个新的集合来存储要保留的方法
        preserved_methods = {
            method
            for method in methods
            if not (hasattr(method, "__module__") and method.__module__ not in config.EXTENSION_MODULES)
        }
        agent_collector.tag_map[tag] = preserved_methods

    try:
        # 遍历 ext_workdir 目录下的所有 Python 文件
        for py_file in ext_workdir.rglob("*.py"):
            # 获取相对路径并转换为模块路径
            rel_path = py_file.relative_to(ext_workdir)
            module_path = str(rel_path.with_suffix("")).replace("/", ".")

            try:
                # 尝试导入并重载模块
                if module_path in sys.modules:
                    module = reload(sys.modules[module_path])
                    if hasattr(module, "clean_up") and callable(module.clean_up):
                        module.clean_up()
                else:
                    module = import_module(module_path)
                reloaded_modules.append(module_path)

                # 检查并添加模块元数据
                if hasattr(module, "__meta__"):
                    module_meta_data = module.__meta__
                    if isinstance(module_meta_data, ExtMetaData):
                        WORKDIR_EXT_META_DATA.append(module_meta_data)
                        logger.success(
                            f'从扩展工作目录加载模块: "{module_meta_data.name}" by "{module_meta_data.author or "Unknown"}" 加载验证成功',
                        )
                    else:
                        logger.warning(f'扩展工作目录模块: "{module_path}" 元数据格式错误')
                else:
                    logger.warning(f'工作目录扩展模块: "{module_path}" 缺少元数据')
            except Exception as e:
                logger.error(f"重载模块失败 {module_path}: {e!s}")
                continue
    except Exception as e:
        logger.error(f"重载 ext_workdir 失败: {e!s}")

    return reloaded_modules



import json
import re
from typing import Any, Dict, Type, TypeVar, Union

from ..exceptions import ComponentRuntimeError
from ..scene import BaseScene, ModelResponse
from .base import BaseComponent

JsonResolverComponentType = TypeVar(
    "JsonResolverComponentType",
    bound="JsonResolverComponent",
)


class JsonResolverComponent(BaseComponent):

    def setup(self, indent: int = 4):
        self._indent = indent
        return self

    @classmethod
    async def render(cls, indent: int = 4) -> str:
        return json.dumps(cls.to_dict(), ensure_ascii=False, indent=indent)

    @classmethod
    async def example(cls, indent: int = 4) -> str:
        """生成响应格式示例 Prompt"""
        ret: str = f"```json\n{await cls.render(indent)}\n```"
        return ret

    @classmethod
    def to_dict(cls) -> Dict:
        """将组件类的字段转化为字典格式"""
        ret = {}
        for attr in dir(cls):
            if (
                not attr.startswith("p_")
                and not attr.startswith("_")
                and not callable(getattr(cls, attr))
                and (attr not in ["scene", "Params", "params"])
            ):
                attr_value = getattr(cls, attr)
                if isinstance(attr_value, JsonResolverComponent) and hasattr(
                    attr_value,
                    "to_dict",
                ):
                    ret[attr] = attr_value.to_dict()  # type: ignore
                elif (  # 基本类型
                    isinstance(attr_value, (str, int, float, bool))
                    or attr_value is None
                ):
                    ret[attr] = attr_value
        return ret

    def resolve_from_text(
        self: JsonResolverComponentType,
        response_text: str,
    ) -> JsonResolverComponentType:
        """解析模型响应"""

        data = withdraw_json(response_text)
        return self.resolve_from_data(data)

    def resolve_from_data(
        self: JsonResolverComponentType,
        data: Any,
    ) -> JsonResolverComponentType:
        """从字典数据中初始化组件"""

        obj = self.__class__(self.scene)
        if isinstance(data, dict):
            for key, value in data.items():
                annotation_type = self.__annotations__.get(key)
                if isinstance(annotation_type, Type):
                    if issubclass(
                        annotation_type,
                        JsonResolverComponent,
                    ):
                        sub_obj = annotation_type(self.scene)
                        setattr(obj, key, sub_obj.resolve_from_data(value))
                    else:
                        setattr(obj, key, value)
                else:
                    raise ComponentRuntimeError(
                        f"Invalid annotation type: {annotation_type}",
                    )
        else:
            raise ComponentRuntimeError(f"Invalid data type: {type(data)}")
        return obj


def withdraw_json(s: str) -> Any:
    """提取返回文本中的 JSON 内容"""

    s = s.strip()

    try:
        s = json.loads(s)
    except json.JSONDecodeError:
        pass
    else:
        return s

    if re.match(r"^```json\n(.*?)\n```$", s, re.DOTALL):
        s = re.match(r"^```json\n(.*?)\n```$", s, re.DOTALL).group(1)  # type: ignore
    elif re.match(r"^```JSON\n(.*?)\n```$", s, re.DOTALL):
        s = re.match(r"^```JSON\n(.*?)\n```$", s, re.DOTALL).group(1)  # type: ignore
    else:
        raise ComponentRuntimeError("Cannot find JSON content in the response.")

    try:
        s = json.loads(s)
    except json.JSONDecodeError as e:
        raise ComponentRuntimeError(f"Invalid JSON format: {e}") from e
    else:
        return s
